/*
 * Copyright 2009 Nicolas LE GRIEL.
 * 
 * Les sources qui constituent ce projet nlegriel-dvp-defi5 de mme que 
 * la documentation associe sont la proprit de leur auteur.
 * 
 * Je donne mon accord au site developpez.com pour l'utilisation de 
 * tout ou partie des sources et de la documentation de ce projet dans 
 * les pages developpez.com
 */
package com.developpez.nlegriel.defi5.view;

import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.util.Enumeration;

import javax.media.j3d.Behavior;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.WakeupOnAWTEvent;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

/**
 * Gestion des mouvements de la camra permettant de visualiser les cubes du
 * puzzle et des pices du puzzle.
 * 
 * @author Nicolas LE GRIEL
 */
public class MoveBehavior extends Behavior {
	public final static double INITIAL_EYE_COTE = 20;
	
	private final static double ROTATION_STEP = Math.PI / 64;
	private final static double SCALE_STEP = 0.05;
	
	private TransformGroup transformGroup;
	
	private Point3d eye = new Point3d(0, 0, INITIAL_EYE_COTE);
	private Point3d center = new Point3d();
	private Vector3d up = new Vector3d(0, 1, 0);

	private Transform3D viewPoint = new Transform3D();
	
	private WakeupOnAWTEvent keyEvent = new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED);

	
	public MoveBehavior(TransformGroup transformGroup) {
		this.transformGroup = transformGroup;
	}

	public void initialize() {
		this.wakeupOn(keyEvent);
	}

	@SuppressWarnings("unchecked")
	public void processStimulus(Enumeration criteria) {
		AWTEvent events[] = keyEvent.getAWTEvent();
		KeyEvent keyEventInfo = (KeyEvent) events[0];
		
		Transform3D transformEye = new Transform3D();

		switch (keyEventInfo.getKeyCode()) {
			case KeyEvent.VK_NUMPAD1:
				transformEye.rotX(ROTATION_STEP);
				break;
			case KeyEvent.VK_NUMPAD3:
				transformEye.rotX(-ROTATION_STEP);
				break;
			case KeyEvent.VK_NUMPAD4:
				transformEye.rotY(ROTATION_STEP);
				break;
			case KeyEvent.VK_NUMPAD6:
				transformEye.rotY(-ROTATION_STEP);
				break;
			case KeyEvent.VK_NUMPAD7:
				transformEye.rotZ(ROTATION_STEP);
				break;
			case KeyEvent.VK_NUMPAD9:
				transformEye.rotZ(-ROTATION_STEP);
				break;
			case KeyEvent.VK_NUMPAD2:
				double distance = eye.distance(center);
				if (distance < 70)
					transformEye.setScale(1 + SCALE_STEP);
				break;
			case KeyEvent.VK_NUMPAD8:
				distance = eye.distance(center);
				if (distance > 15)
					transformEye.setScale(1 - SCALE_STEP);
				break;
			default:
				this.wakeupOn(keyEvent);
				return;
		}

		transformEye.transform(eye);
		
		viewPoint.lookAt(eye, center, up);
		viewPoint.invert();
		transformGroup.setTransform(viewPoint);

		this.wakeupOn(keyEvent);
	}
}
